library props_flutter_notificationsdk;

import 'dart:async';
import 'dart:convert';

import 'package:device_info_plus/device_info_plus.dart';
import 'package:firebase_core/firebase_core.dart';
import 'package:flutter/services.dart';
import 'package:flutter_local_notifications/flutter_local_notifications.dart';
import 'package:http/http.dart' as http;
import 'package:firebase_messaging/firebase_messaging.dart';


class PropsFlutterNotificationsdk {
  String? coriginKeys;
  String? payload;
  late FirebaseMessaging _fbm;
  late Future _doneFuture;
  Future get initDone => _doneFuture;

  late Timer _timer;

  void processTimer() {
    _timer = Timer.periodic(Duration(minutes: 1), (timer) {
      PropsFlutterNotificationsdk.instance.callEvents("spend one minutes");
    });
  }

  Future<void> firebaseMessagingBackgroundHandler(RemoteMessage message) async {
    // If you're going to use other Firebase services in the background, such as Firestore,
    // make sure you call `initializeApp` before using other Firebase services.
    await Firebase.initializeApp();
    String id = message.data["props_notifid"] ?? '';
    PropsFlutterNotificationsdk.instance.logMessageReceived(id);
    print("Handling a background message: ${message.messageId}");
  }

  Future _initFb() async {
    // FirebaseMessaging.onBackgroundMessage(firebaseMessagingBackgroundHandler);
    await Firebase.initializeApp().whenComplete(() => {
      print("firebase initialized")
    });
    _fbm = FirebaseMessaging.instance;
    FirebaseMessaging.instance.setForegroundNotificationPresentationOptions(
      alert: true,
      badge: true,
      sound: true,
    );
    FirebaseMessaging.onMessage.listen((event) {
      print('Got a message whilst in the foreground!');
      print('Message data: ${event.data}');

      if (event.notification != null) {
        PropsFlutterNotificationsdk.instance.showNotification(event.notification!.title!, event.notification!.body!, event.data);
        print('Message also contained a notification: ${event.notification}');
      }
    });
    await FirebaseMessaging.instance.subscribeToTopic('all');
    processTimer();
    await initializePushnotif();
  }
  
  PropsFlutterNotificationsdk._internal(){
    print("creating firebase ");
    _doneFuture = _initFb();
  }
  static final PropsFlutterNotificationsdk instance = PropsFlutterNotificationsdk._internal();
  factory PropsFlutterNotificationsdk() {
    return instance;
  }

  final FlutterLocalNotificationsPlugin flutterLocalNotificationsPlugin =
    FlutterLocalNotificationsPlugin();

  MethodChannel platform =
    MethodChannel('dexterx.dev/flutter_local_notifications_example');

  Future<void> initializePushnotif() async {
    const AndroidInitializationSettings initializationSettingsAndroid =
      AndroidInitializationSettings("@mipmap/ic_launcher");
      final InitializationSettings initializationSettings = InitializationSettings(
        android: initializationSettingsAndroid,
      );

    await flutterLocalNotificationsPlugin.initialize(initializationSettings,
      onSelectNotification: (String? payload) async {
      if (payload != null) {
        print('notification payload: $payload');
        Map<String, dynamic> map = jsonDecode(payload);
        PropsFlutterNotificationsdk.instance.logMessageOpened(map['props_notifid']);
      }
      this.payload = payload;
    });
  }

  void showNotification(String title, String body, Object pl) async {
    const AndroidNotificationDetails androidPlatformChannelSpecifics =
    AndroidNotificationDetails('propsChannelID', 'Props Notification Service',
        channelDescription: 'your channel description',
        importance: Importance.max,
        priority: Priority.high,
        ticker: 'ticker');
    const NotificationDetails platformChannelSpecifics =
    NotificationDetails(android: androidPlatformChannelSpecifics);
      await flutterLocalNotificationsPlugin.show(
    0, title, body, platformChannelSpecifics,
    payload: jsonEncode(pl));
  }

  void setCoriginKeys(String coriginKeys) {
    this.coriginKeys = coriginKeys;
  }

  String? getCoriginKeys() {
    return this.coriginKeys;
  }

  Future<void> registerUserData() async {
    DeviceInfoPlugin deviceInfo = DeviceInfoPlugin();
    AndroidDeviceInfo androidInfo = await deviceInfo.androidInfo;

    _fbm.getToken().then((token) async {
      var getIPInfoUrl = "https://ipinfo.io/json";
      var IPurl = Uri.parse(getIPInfoUrl);
      var getData = await http.get(IPurl);
      var res = jsonDecode(getData.body);
      String city = res['city'];
      String region = res['region'];
      String country = res['country'];
      var fullUrl = "https://apiapps.props.id/registerUser?deviceId=${androidInfo.model}&city=$city&region=$region&country=$country&brand=${androidInfo.brand}&uuid=${androidInfo.androidId}&messagingToken=$token";
      var url = Uri.parse(fullUrl);
      Map<String, String> headers = {
        "Authorization": PropsFlutterNotificationsdk.instance.getCoriginKeys()!
      };
      var test = await http.get(url, headers: headers);
      return;
    });
  }

  Future<void> logMessageReceived(String notificationId) async {
    DeviceInfoPlugin deviceInfo = DeviceInfoPlugin();
    AndroidDeviceInfo androidInfo = await deviceInfo.androidInfo;
    print('Running on ${androidInfo.model}');  // e.g. "Moto G (4)"

    _fbm.getToken().then((token) async {
      var fullUrl = "https://apiapps.props.id/logMessageReceived?notificationId=${notificationId}&uuid=${androidInfo.androidId}";
      var url = Uri.parse(fullUrl);
      Map<String, String> headers = {
        "Authorization": PropsFlutterNotificationsdk.instance.getCoriginKeys()!
      };
      var test = await http.get(url, headers: headers);
      return;
    });
  }

  Future<void> logMessageOpened(String notificationId) async {
    DeviceInfoPlugin deviceInfo = DeviceInfoPlugin();
    AndroidDeviceInfo androidInfo = await deviceInfo.androidInfo;
    print('Running on ${androidInfo.model}');  // e.g. "Moto G (4)"

    _fbm.getToken().then((token) async {
      var fullUrl = "https://apiapps.props.id/logMessageOpen?notificationId=${notificationId}&uuid=${androidInfo.androidId}";
      print('Calling ${fullUrl}');
      print('get auth ${PropsFlutterNotificationsdk.instance.getCoriginKeys()}');
      var url = Uri.parse(fullUrl);
      Map<String, String> headers = {
        "Authorization": PropsFlutterNotificationsdk.instance.getCoriginKeys()!
      };
      var test = await http.get(url, headers: headers);
      return;
    });
  }

  Future<void> callEvents(String notificationId) async {
    DeviceInfoPlugin deviceInfo = DeviceInfoPlugin();
    AndroidDeviceInfo androidInfo = await deviceInfo.androidInfo;

    _fbm.getToken().then((token) async {
      var fullUrl = "https://apiapps.props.id/callEvent?eventId=${notificationId}&uuid=${androidInfo.androidId}";
      var url = Uri.parse(fullUrl);
      Map<String, String> headers = {
        "Authorization": PropsFlutterNotificationsdk.instance.getCoriginKeys()!
      };
      var test = await http.get(url, headers: headers);
      return;
    });
  }
}
